#include "httpd.h"
#include "http_log.h"
#include "http_config.h"
#include "http_core.h"
#include "ap_alloc.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libpq-fe.h>
#include "pglogger.h"

#define PGLOGGER_VERSION "mod_pglogger 1.01 (c) 2003 GeekNET"

static enum {
	PGLOGGER_IP,
	PGLOGGER_HOSTNAME
} pglogger_logstyle;

typedef struct {
	char *dbstring;
	int logstyle;
} pglogger_server_cfg;

module MODULE_VAR_EXPORT pglogger_module;

static void pglogger_server_init(server_rec * c, pool * p);
static void *pglogger_server_config(pool * p, server_rec * c);
static int pglogger_log(request_rec * r);
static void pglogger_child_init(server_rec * c, pool * p);
static void pglogger_child_exit(server_rec * c, pool * p);

static const char *pglogger_set_option(cmd_parms * cmd, void *dummy, char *arg);
static const char *pglogger_set_logstyle(cmd_parms * cmd, void *dummy, char *arg);

static command_rec pglogger_cmds[] = {
	{"PGloggerDBString", pglogger_set_option, NULL,
	RSRC_CONF, TAKE1, "pglogger database connection object"},
	{"PGloggerLogStyle", pglogger_set_logstyle, NULL,
	RSRC_CONF, TAKE1, "pglogger logstyle"},
	{NULL}
};

static const char *pglogger_set_option(cmd_parms * cmd, void *dummy, char *arg)
{
	server_rec *s = cmd->server;

	pglogger_server_cfg *cfg = (pglogger_server_cfg *)
		ap_get_module_config(s->module_config, &pglogger_module);

	cfg->dbstring = strdup(arg);
	return NULL;
}

static const char *pglogger_set_logstyle(cmd_parms * cmd, void *dummy, char *arg)
{
	server_rec *s = cmd->server;
	pglogger_server_cfg *cfg = (pglogger_server_cfg *)
		ap_get_module_config(s->module_config, &pglogger_module);
	if (strcmp("ip", arg) == 0) cfg->logstyle = PGLOGGER_IP;
	else 
	if (strcmp("hostname", arg) == 0) cfg->logstyle = PGLOGGER_HOSTNAME;
	else
	return "No valid argument for PGloggerLogStyle";

	return NULL;
}

module MODULE_VAR_EXPORT pglogger_module = {
	STANDARD_MODULE_STUFF,
	pglogger_server_init,	/* initializer */
	NULL,			/* dir config creater */
	NULL,			/* dir merger --- default is to override */
	pglogger_server_config,	/* server config */
	NULL,			/* merge server config */
	pglogger_cmds,		/* command table */
	NULL,			/* handlers */
	NULL,			/* filename translation */
	NULL,			/* check_user_id */
	NULL,			/* check auth */
	NULL,			/* check access */
	NULL,			/* type_checker */
	NULL,			/* fixups */
	pglogger_log,		/* logger */
	NULL,			/* header parser */
	pglogger_child_init,	/* child_init */
	pglogger_child_exit,	/* child_exit */
	NULL			/* post read-request */
};

static void pglogger_server_init(server_rec * c, pool * p)
{
	ap_add_version_component("PGLogger/" PGLOGGER_VERSION);
}

static void *pglogger_server_config(pool * p, server_rec * c)
{
	pglogger_server_cfg *cfg = (pglogger_server_cfg *) ap_pcalloc(p, sizeof *cfg);
	cfg->logstyle = PGLOGGER_IP;
	return cfg;
}

static int pglogger_log(request_rec * r)
{
	struct tm *loctime;
	log values;

	pglogger_server_cfg *cfg = (pglogger_server_cfg *) ap_get_module_config(r->server->module_config,&pglogger_module);

	switch ( cfg->logstyle ) {
		case PGLOGGER_IP:
			strncpy(values.rclient,r->connection->remote_ip,255);
			break;
		case PGLOGGER_HOSTNAME:
			strncpy(values.rclient,ap_get_remote_host(r->connection, r->per_dir_config, 1 ),255);
			break;
	}

	if(ap_get_remote_logname(r) != NULL)
	    strncpy(values.rlogname,ap_get_remote_logname(r),32);
	else
	    strcpy(values.rlogname,"-");

	if(r->connection->user == NULL || strlen(r->connection->user) == 0)
	    strcpy(values.ruser,"-");
	else
	    strncpy(values.ruser,r->connection->user,32);
	
	if(ap_table_get(r->headers_in, "Referer") != NULL)
	    strncpy(values.referer,ap_table_get(r->headers_in, "Referer"),255);
	else
	    strcpy(values.referer,"-");
	
	if(ap_table_get(r->headers_in, "User-Agent") != NULL)
	    strncpy(values.clientsoftware,ap_table_get(r->headers_in, "User-Agent"),255);
	else
	    strcpy(values.clientsoftware,"-");

	if(r->the_request != NULL)
	    strncpy(values.the_request,r->the_request,255);
	else
	    strcpy(values.the_request,"");

	if(r->server->server_hostname != NULL)
	    strncpy(values.server_hostname,r->server->server_hostname,255);
	else
	    strcpy(values.server_hostname,"");

	values.statuscode = r->status;
	values.bytessenttoclient = r->bytes_sent;
	values.timespent = time(NULL) - r->request_time;
	
	loctime = localtime (&(r->request_time));
	strftime( values.time_buffer, TIME_BUFSIZE, "[%d/%b/%Y:%H:%M:%S %z]", loctime);

	log2db(&values);
	return OK;
}

static void pglogger_child_init(server_rec * c, pool * p)
{
	pglogger_server_cfg *cfg;

	cfg = (pglogger_server_cfg *) ap_get_module_config(c->module_config,
		&pglogger_module);

	opendb(cfg->dbstring);
}

static void pglogger_child_exit(server_rec * c, pool * p)
{
	pglogger_server_cfg *cfg;

	cfg = (pglogger_server_cfg *) ap_get_module_config(c->module_config,
		&pglogger_module);

	free(cfg->dbstring);
	closedb();
}
